@@ -187,9 +187,11 @@ def get_block_optimized(diP):
     # Return list of positions for the block with highest mean
 	return li_interval_possible[0][0]
 
-
+############################################################
 def search_block_optimized(diP, text, threshold):
-	""" Search of a set of subwords through a text using Aho-Corasick algorithm for a given threshold and check the windows by using LAM. The subwords are compute with a heuristic.
+	"""Find in the sequence <text>, all occurrences of the di-PWM <diP> whose score is greater than the minimum score threshold <threshold>. Done using a core block optimized strategy: the core block is a reduced di-PWM with selective positions. The search is done for the core block and core occurrences are extended to full length word occurrences. See the article for details.
+
+           Other description: Search of a set of subwords through a text using Aho-Corasick algorithm for a given threshold and check the windows by using LAM. The subwords are compute with a heuristic.
 
 	Args:
 		diP (diPWM): object diPWM
@@ -201,12 +203,22 @@ def search_block_optimized(diP, text, threshold):
 	Yields:
 		tuple: starting_position, word, score
 	"""
+        # compute the core block of the di-PWM
+        # this defines - a reduced di-PWM containing only the columns for core
+        #              - and a core length that is shorter compared to the initial di-PWM
+        
 	block = get_block_optimized(diP)
 
+        # enumerate the core valid words for the core length and build an Aho_Corasick automaton for them
+
 	autom = ahocorasick.Automaton()
 	for word, score in enumerate_block(diP, block, threshold):
 		autom.add_word(word, (word,score))
 	autom.make_automaton()
+        
+        # search for occurrences of core valid words
+        # then check the window centered on the core valid word
+        # to verify if there is a full-length occurrence
 
 	for (position, (word, score)) in autom.iter(text):
 		start_position = position - (block[-1] + 1)
@@ -215,9 +227,14 @@ def search_block_optimized(diP, text, threshold):
 			if score:
 				yield start_position,text[start_position:start_position+diP.length], score
 
-
+############################################################
 def search_block_optimized_ratio(diP, text, ratio):
-	""" Search of a set of subwords through a text using Aho-Corasick algorithm for a given threshold and check the windows by using LAM. The subwords are compute with a heuristic. From the ratio is calculated the threshold.
+	""" Find occurrences of the di-PWM <diP> in the sequence <text> for a ratio threshold <ratio>
+            1. computes a score threshold from the ratio
+            2. call function search_block_optimized to perform the search of the occurrences in the sequence
+
+	Other description:
+              Search of a set of subwords through a text using Aho-Corasick algorithm for a given threshold and check the windows by using LAM. The subwords are compute with a heuristic. From the ratio is calculated the threshold.
 
 	Args:
 		diP (diPWM): object diPWM
@@ -235,4 +252,37 @@ def search_block_optimized_ratio(diP, text, ratio):
 		yield position, word, score
 
 
+############################################################
+# fn added 10.02.23
+# good default pvalue threshold ? here 10^-6
+
+def search_block_optimized_pvalue(diP, text, pvalue = 0.000001 ):
+	""" Find occurrences of the di-PWM <diP> in the sequence <text>  for the pvalue threshold <pvalue>
+            1. computes a score threshold from the pvalue
+            2. call function search_block_optimized to perform the search of the occurrences in the sequence
+
+        Remark: it is difficult to find a good pvalue threshold ; we advise to make several trials starting with very low (restrictive) threshold values. Otherwise, use the function search_block_optimized_ratio instead.
+
+	Other description:
+              Search of a set of subwords through a text using Aho-Corasick algorithm for a given threshold and check the windows by using LAM. The subwords are compute with a heuristic. From the ratio is calculated the threshold.
+
+	Args:
+		diP (diPWM): object diPWM
+
+		text (string): text to search on the motif (first position = 0)
+
+		pvalue (float): pvalue threshold given to select the occurrences.
+                                Between 0 and strictly less to 1.
+
+	Yields:
+		tuple: starting_position, word, score
+	"""
+        
+	# calculate threshold from pvalue
+	threshold = diP.set_threshold_from_pvalue( pvalue )
+        # perform the search
+	for position, word, score in search_block_optimized(diP, text, threshold):
+		yield position, word, score
+
+
 #
